/*

  Glt OpenGL C++ Toolkit (Glt)
  Copyright (C) 2000-2001 Nigel Stewart
  Email: nigels@nigels.com   WWW: http://www.nigels.com/glt/

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <glutm/main.h>
#include <glutm/winexam.h>
#include <glutm/glut.h>
#include <glutm/shape.h>

#include <glt/fontasci.h>
#include <glt/rgb.h>

#include <node/shape.h>
#include <node/text.h>

#include <fonts/fonts.h>

#include <memory>
#include <iostream>
using namespace std;

class GltShape;

class ExaminerDemo : public GlutWindowExaminer
{
public:
	ExaminerDemo(		
		int width                = widthDefault, 
		int height               = heightDefault,
		int x                    = xDefault, 
		int y                    = yDefault
	);
   ~ExaminerDemo();

   void OnOpen();
   void OnClose();
   void OnDisplay();
   void OnTick();
   void OnKeyboard(unsigned char key, int x, int y);

   static const string _message;

protected:

	GltFontAscii			 _font;
	GltTextOverlay			 _info;
	std::auto_ptr<GltShape>  _shape;
};

const string ExaminerDemo::_message =

	"GlutMaster Examiner Demo.           \n" \
	"\n"\
	"Use mouse buttons to rotate, zoom and pan. \n" \
	"Keys 0 to 9 select different objects.      \n";


ExaminerDemo::ExaminerDemo(int width,int height,int x,int y)
: GlutWindowExaminer("GlutMaster Examiner Demo",width,height,x,y),
  _info(_font,_viewport)
{
}

ExaminerDemo::~ExaminerDemo()
{
}

void
ExaminerDemo::OnOpen()
{
	_font.init(vga8Font);

	string2wstring(_info.text(),_message);

	glEnable(GL_DEPTH_TEST);
	_viewMatrix = matrixRotate(Vector1,60.0)*_viewMatrix;

	OnKeyboard('1',0,0);	// Select sphere to start with
	setTick(1000/60);		// 60fps maximum
}

void
ExaminerDemo::OnClose()
{
	_font.init(NULL);
}

void ExaminerDemo::OnDisplay()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   if (_shape.get())
	   _shape->draw();

   _info.draw();
}

void ExaminerDemo::OnTick()
{
	if (!mousePressed())
		_viewMatrix = matrixRotate(Vector1,0.2)*_viewMatrix;

	postRedisplay();
}

void ExaminerDemo::OnKeyboard(unsigned char key, int x, int y)
{
	GlutWindowExaminer::OnKeyboard(key,x,y);

	GltShape *newShape = NULL;

	switch (key)
	{
	case '1': newShape = new GlutSphere(false);        break;
	case '2': newShape = new GlutCube(false);          break;
	case '3': newShape = new GlutCylinder(false);      break;
	case '4': newShape = new GlutCone(false);          break;
	case '5': newShape = new GlutTorus(false);         break;
	case '6': newShape = new GlutDodecahedron(false);  break;
	case '7': newShape = new GlutOctahedron(false);    break;
	case '8': newShape = new GlutTetrahedron(false);   break;
	case '9': newShape = new GlutIcosahedron(false);   break;
	case '0': newShape = new GlutTeapot(false);        break;
	}

	if (newShape)
	{
		// gcc doesn't allow:  _shape = auto_ptr<GltShape>(newShape)

		auto_ptr<GltShape> tmp(newShape);
		_shape = tmp;
	}

	_shape->color() = red;
	_shape->solid() = false;

	postRedisplay();
}

bool GlutMain(const std::vector<std::string> &arg)
{
	cout << ExaminerDemo::_message << endl;
	
	GlutWindow *main = new ExaminerDemo(400,400,20,20);
	main->open();

	return true;
}
