#include <glutm/master.h>
#include <glutm/winexam.h>

#include <node/skybox.h>

#include <glt/error.h>

#include <iostream>
using namespace std;

#include "textures.h"


class SkyBoxWindow : public GlutWindowExaminer
{
public:
	SkyBoxWindow
	(
		int width                = widthDefault, 
		int height               = heightDefault,
		int x                    = xDefault, 
		int y                    = yDefault
	);

   ~SkyBoxWindow();

	void OnOpen();
	void OnClose();
	void OnDisplay();
	void OnKeyboard(unsigned char key, int x, int y);

private:

	GltSkyBox			_skyBox;
	bool				_showWire;
	int                 _current;

	void setCurrent();
};

SkyBoxWindow::SkyBoxWindow(int width,int height,int x,int y)
: GlutWindowExaminer("GLT SkyBox Demo",width,height,x,y,GLUT_DOUBLE|GLUT_RGBA|GLUT_DEPTH),
  _current(0), _showWire(false)
{ 
}

SkyBoxWindow::~SkyBoxWindow()
{
}

void
SkyBoxWindow::setCurrent()
{
	_skyBox.clear();

	switch (_current)
	{
		/* Blue Sky textures from OpenGL Feathers */

	case 0:
		_skyBox.negativeX().init(blueskynxTexture);
		_skyBox.negativeY().init(blueskynyTexture);
		_skyBox.negativeZ().init(blueskynzTexture);
		_skyBox.positiveX().init(blueskypxTexture);
		_skyBox.positiveY().init(blueskypyTexture);
		_skyBox.positiveZ().init(blueskypzTexture);
		break;

		/* Hills textures from NVIDIA SDK */

	case 1:
		_skyBox.negativeX().init(hillsnxTexture);
		_skyBox.negativeY().init(hillsnyTexture);
		_skyBox.negativeZ().init(hillsnzTexture);
		_skyBox.positiveX().init(hillspxTexture);
		_skyBox.positiveY().init(hillspyTexture);
		_skyBox.positiveZ().init(hillspzTexture);
		break;
	}
}

void 
SkyBoxWindow::OnOpen()
{
	setCurrent();
}

void 
SkyBoxWindow::OnClose()
{
	_skyBox.clear();
}

void 
SkyBoxWindow::OnDisplay()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	_skyBox.solid() = true;
	_skyBox.draw();

	if (_showWire)
	{
		_skyBox.solid() = false;
		_skyBox.draw();
	}

	GLERROR;
}

void 
SkyBoxWindow::OnKeyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
	case ' ':
		_current = (_current+1)%2;
		setCurrent();
		break;

	case 'w':
		_showWire = !_showWire;
		break;

	default:
		GlutWindowExaminer::OnKeyboard(key,x,y);
		return;
	}

	postRedisplay();
}

#include <glutm/main.h>

bool GlutMain(const std::vector<std::string> &arg)
{
	GlutWindow *main = new SkyBoxWindow(400,400,20,20);
	main->open();

	return true;
}
